﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using Kooboo.Globalization;
using Kooboo.CMS.Sites;
using Kooboo.CMS.Sites.Extension.ModuleArea;
using Kooboo.CMS.DataModule.Models;

namespace Kooboo.CMS.DataModule.Controllers
{
    public class AdminController : AdminControllerBase
    {
        public ActionResult Index(string siteName)
        {
            ModuleInfo_Metadata moduleInfo = new ModuleInfo_Metadata(ModuleName, siteName);
            return View(moduleInfo);
        }

        [HttpPost]
        public ActionResult Index(string siteName, ModuleInfo_Metadata moduleInfo)
        {
            JsonResultEntry resultEntry = new JsonResultEntry(ModelState);
            try
            {
                if(ModelState.IsValid)
                {
                    ModuleInfo.SaveModuleSetting(ModuleName, siteName, moduleInfo.Settings);
                    resultEntry.AddMessage("Module setting has been changed.".Localize());
                }

            }
            catch(Exception e)
            {
                resultEntry.AddException(e);
            }
            return Json(resultEntry);
        }

        public ActionResult InitializeModule(string siteName)
        {
            return View();
        }

        public ActionResult GenerateModuleInfo()
        {
            ModuleInfo moduleInfo = new ModuleInfo();
            moduleInfo.ModuleName = "SampleModule";
            moduleInfo.Version = "3.1.0.0";
            moduleInfo.KoobooCMSVersion = "3.1.0.0";
            moduleInfo.DefaultSettings = new ModuleSettings()
            {
                ThemeName = "Default",
                Entry = new Entry()
                {
                    Controller = "Home",
                    Action = "Index"
                }
            };
            moduleInfo.EntryOptions = new EntryOption[]{
                new EntryOption(){ Name="NewsList",Entry = new Entry{ Controller="Home",Action ="Index"}},
                new EntryOption(){ Name="NewsCategories",Entry = new Entry{ Controller="Home",Action ="Categories"}},
                new EntryOption(){Name="ArticleCategories",Entry=new Entry{Controller="Article",Action="Categories"}},
                new EntryOption(){Name="ArticleList",Entry=new Entry{Controller="Article",Action="List"}},
            };
            moduleInfo.DefaultSettings.CustomSettings = new Dictionary<string, string>();
            moduleInfo.DefaultSettings.CustomSettings["Setting1"] = "Value1";
            ModuleInfo.Save(moduleInfo);

            return View();
        }
    }
}